#!/usr/bin/python3
# Old McDonnold's Farm - a song by Mike Cook
 
from rc522rfid import Rc522rfid
import pygame, time, os

pygame.init()                   # initialise graphics interface
pygame.mixer.quit()
pygame.mixer.init(frequency=22050, size=-16, channels=2, buffer=512)
os.environ['SDL_VIDEO_WINDOW_POS'] = 'center'
pygame.display.set_caption("Old McDonald's Farm")
pygame.event.set_allowed(None)
pygame.event.set_allowed([pygame.KEYDOWN,pygame.QUIT])
screenWidth = 723 ; screenHeight = 369
screen = pygame.display.set_mode([screenWidth,screenHeight],0,32)

rfidReader = Rc522rfid()

farm = [] # what animals are on the farm today
play = False
cardsRead = -1
verse = 0

def main():
   global play,xPlot,yPlot,cardsRead,farm
   print ("Old McDonald's Farm")
   print ("Present cards then Press - p = Play - n = New")
   init()
   while 1:
      drawScreen(-1,0,0)
      while not play:
         checkForEvent()
         if readCard() == 0:
            print("Farm now contains ",end="")
            for i in range(0,len(farm)):
               print(rawNames[farm[i]]," ",end="")
            print()   
            rfidReader.RC522_WaitForCardRemoved()
      singSong()
      play = False
      farm = [] ; xPlot = [] ; yPlot = []
      cardsRead = -1
      print("Place cards to populate farm")
   
def drawScreen(n,level,ind):
   pygame.draw.rect(screen,(255,255,255),(0,0,screenWidth,screenHeight),0)
   screen.blit(farmBuilding,(0,0))
   if cardsRead != -1 and n != -1:
      if verse  > 0 and level >0: # draw previous animals
         i = verse
         while i > ind:
           k = farm[i]
           for j in range(0,5):
              screen.blit(anamalPictures[k],(xPlot[i][j],yPlot[i][j]))
           i -=1           
      k = farm[ind]
      for j in range(0,n): # draw latest anamal
         screen.blit(anamalPictures[k],(xPlot[ind][j],yPlot[ind][j]))
         
   screen.blit(farmFence,(0,270))
   pygame.display.update()

def readCard():
   status = rfidReader.RC522_ReadCard()
   if status == -1:
      return -1
   # This is the (default) key for authentication
   key = [0xFF,0xFF,0xFF,0xFF,0xFF,0xFF]
   block1 = rfidReader.RC522_GetSector(key,12)
   block2 = rfidReader.RC522_GetSector(key,13)
   block3 = rfidReader.RC522_GetSector(key,14)
   if block1 != -1 and block2 != -1 and block3 != -1: # if data OK
      lookAtNew(block1,block2,block3)
      return 0
   else:
      print("Error in reading card")
      rfidReader.RC522_WaitForCardRemoved()

def lookAtNew(s1,s2,s3): # look at the new element
   global xPlot,yPlot, cardsRead
   name = ""
   i = 0
   notDone = True
   while notDone and i < 16: # generate name
      c = s1[i]
      i+=1
      if c !=0:     
        name = name + chr(c)
      else:
        notDone = False
   for i in range(0,len(animalNames)):
      if animalNames[i] == name: # find the name on the card
         farm.append(i)
         xPlot.append([0,0,0,0,0])
         yPlot.append([0,0,0,0,0])
         cardsRead += 1
         k = 0
         for j in range(0,4):
            xPlot[cardsRead][j] = s2[k]<< 8 | s2[k+1]
            yPlot[cardsRead][j] = s2[k+2]<< 8 | s2[k+3]
            k += 4
         xPlot[cardsRead][4] = s3[0]<< 8 | s3[1]
         yPlot[cardsRead][4] = s3[2]<< 8 | s3[3]           
               
def init():
   global animalNames,anamalPictures,farmBuilding,farmFence,animals
   global xPlot,yPlot, noises, rawNames, fixed
   path = os.path.realpath(__file__) 
   path = os.path.dirname(path) + "/farmPictures/"
   #get a list of files
   animalNames = [fn for fn in next(os.walk(path))[2]]
   list.sort(animalNames) # put in alphabetical order
   rawNames = []
   for i in range (0,len(animalNames)): # remove file extension
      rawNames.append(animalNames[i][0:len(animalNames[i])-4])   
   #load in the sounds   
   animals = [ pygame.mixer.Sound("farmSounds/"+rawNames[sound]+".ogg")
                  for sound in range(0,len(rawNames))]
   noises = [ pygame.mixer.Sound("farmSounds/"+rawNames[sound]+"Sound.ogg")
                  for sound in range(0,len(rawNames))]
   files = [ "start","and","ee_eye_oh","witha","here","anda","there","herea","therea","everywherea","end","well" ]
   fixed = [ pygame.mixer.Sound("farmSounds/fixed/"+files[sound]+".ogg")
                  for sound in range(0,len(files))]
   #load in the pictures
   anamalPictures= [ pygame.image.load("farmPictures/"+animalNames[i]).convert_alpha()
                  for i in range(0,len(animalNames))]
   farmBuilding = pygame.image.load("farmPictures/fixed/farm.png").convert_alpha()
   
   farmFence = pygame.image.load("farmPictures/fixed/fence.png").convert_alpha()
   xPlot = [] ; yPlot = []
         
def singSong():
   global verse
   verse = 0
   while verse < len(farm): # repeat for each verse
      for i in range(0,3): # start part
         fixed[i].play()
         waitFinish()
         if i == 1:
           animals[farm[verse]].play()
           waitFinish()
         if i == 2:
            farmYard(verse,verse)
            verse += 1            
   
def farmYard(index,verse): # sing verse
   numDisplayed = 0
   level = 0
   while index != -1:
      for i in range(3,10):
         fixed[i].play()
         waitFinish()
         if i == 3 or i==5 or i==9:
            noises[farm[index]].play()
            numDisplayed +=1
            drawScreen(numDisplayed,level,index)
            waitFinish()
            noises[farm[index]].play()
            waitFinish()
         if i == 7 or i == 8:
            noises[farm[index]].play()
            numDisplayed +=1
            drawScreen(numDisplayed,level,index)            
            waitFinish()
      index -=1
      level +=1
      numDisplayed = 0
   if verse < len(farm)-1: # more verses
      fixed[0].play()
      waitFinish()
      time.sleep(0.2)
      fixed[11].play() # well
      drawScreen(-1,0,0) # clear the farm
   else:               # end of song     
      fixed[10].play()
   waitFinish()   
   time.sleep(0.3)
   
def waitFinish():
   while pygame.mixer.get_busy():
      checkForEvent()

def terminate(): # close down the program
    print ("Closing down please wait")
    pygame.mixer.quit()
    pygame.quit() # close pygame
    os._exit(1)
 
def checkForEvent(): # see if we need to quit
   global play, farm, cardsRead
   event = pygame.event.poll()
   if event.type == pygame.QUIT :
         terminate()
   if event.type == pygame.KEYDOWN :
      if event.key == pygame.K_ESCAPE :
          terminate()
      if event.key == pygame.K_p :
          play = True
      if event.key == pygame.K_n :
          farm = []
          cardsRead = -1  
   
# Main program logic:
if __name__ == '__main__':
    main()

            
